@@ -16,7 +16,7 @@ import requests
 from bs4 import BeautifulSoup
 from time import sleep
 from io import StringIO
-from pycliflo.datasets_mapping import Datasets
+from pycliflo.datasets_mapping import Datasets, freq_codes
 # from selenium.common.exceptions import TimeoutException
 # from selenium.webdriver.support.ui import WebDriverWait
 # from selenium.webdriver.support import expected_conditions as EC
@@ -258,6 +258,19 @@ class Cliflo(object):
 
         sleep(2)
 
+        ## Remaining rows
+        try:
+            s_text = 'Remaining rows (before subscription renewal): '
+            rr_text = driver.find_element_by_xpath('//div[contains(text(), "Remaining rows (before subscription renewal): ")]').text
+        except:
+            s_text = 'Remaining rows: '
+            rr_text = driver.find_element_by_xpath('//div[contains(text(), "Remaining rows: ")]').text
+
+        rr = int(rr_text.split(s_text)[1].split('.')[0].replace(',', ''))
+        self.remaining_rows = rr
+        print('Remaining rows: ' + str(rr))
+
+        ## Switch back to data window
         driver.switch_to.window(driver.window_handles[1])
 
         self.driver = driver
@@ -302,89 +315,113 @@ class Cliflo(object):
         agents_box.clear()
         agents_box.send_keys(site_str)
 
-        ### Assign dates
-        self._date_range(driver, from_date, to_date)
-
         ### Assign File format
         format_box = driver.find_element_by_xpath("//option[@value='texttab']")
         format_box.click()
 
-        ### Submit!
-        submit_box = driver.find_element_by_name('submit_sq')
-        submit_box.click()
+        ### Cliflo has a limit of 40000 rows...need to chunk through them...
+        freq = freq_codes[dataset_code]
 
-        print('Please wait for the sites page to load...')
-        sleep(2)
+        dates1 = pd.date_range(from_date, to_date, freq=freq)
+        n_chunks = np.ceil(len(dates1) / float(36000))
+        dates2 = np.array_split(dates1, n_chunks)
 
-        ### Get the data
-        timer = int(timeout/3)
-        while timer > 0:
-            try:
-                page = driver.page_source
-                if len(page) > 20:
-                    break
-            except:
-                timer = timer - 1
-                if timer == 0:
-                    driver.quit()
-                    raise ValueError('Page took too long to load. Please reduce the request size.')
-                sleep(3)
+        ts_list = []
+        for d in dates2:
 
-        driver.back()
-        sleep(1)
+            if self.remaining_rows == 0:
+                sites1 = pd.DataFrame()
+                print('No rows left to get ts data. Get a new subscription.')
+                break
 
-        self.page_source = page
+            ### Assign dates
+            from_date1 = d.min()
+            to_date1 = d.max()
+            self._date_range(driver, from_date1, to_date1)
 
-        ## Sites
-        s_index1 = page.find('Name')
-        s_index2 = page.find('Note: ')
-        sites_str = StringIO(page[s_index1:(s_index2 - 1)])
+            ### Submit!
+            submit_box = driver.find_element_by_name('submit_sq')
+            submit_box.click()
 
-        sites1 = pd.read_table(sites_str)
+            print('Please wait for the sites page to load...')
+            sleep(2)
 
-        self.sites = sites1
+            ### Get the data
+            timer = int(timeout/3)
+            while timer > 0:
+                try:
+                    page = driver.page_source
+                    if len(page) > 20:
+                        break
+                except:
+                    timer = timer - 1
+                    if timer == 0:
+                        driver.quit()
+                        raise ValueError('Page took too long to load. Please reduce the request size.')
+                    sleep(3)
 
-        ## TS data
-        rc_index = page.find('Total number of rows output')
+            driver.back()
+            sleep(1)
 
-        page_ts = page[s_index2:(rc_index - 1)]
+            self.page_source = page
 
-        if page_ts == '':
-            raise ValueError('No useful time series returned')
+            ## Sites
+            s_index1 = page.find('Name')
+            s_index2 = page.find('Note: ')
+            sites_str = StringIO(page[s_index1:(s_index2 - 1)])
 
-        site_index1 = page_ts.find('Station')
-        site_index2 = page_ts.find('Station', site_index1+1)
-        if site_index2 == -1:
-            ts_str = page_ts[site_index1:]
-            footer = 1
+            sites1 = pd.read_table(sites_str)
+
+            self.sites = sites1
+
+            ## TS data
+            rc_index = page.find('Total number of rows output')
+
+            page_ts = page[s_index2:(rc_index - 1)]
+
+            if page_ts == '':
+                raise ValueError('No useful time series returned')
+
+            site_index1 = page_ts.find('Station')
+            site_index2 = page_ts.find('Station', site_index1+1)
+            if site_index2 == -1:
+                ts_str = page_ts[site_index1:]
+                footer = 1
+            else:
+                ts_str = page_ts[site_index1:(site_index2 - 1)]
+                footer = 2
+
+            ts_str1 = StringIO(ts_str)
+            ts1 = pd.read_table(ts_str1, skipfooter=footer, engine='python')
+            if ts1.empty:
+                raise IndexError('No time series data was returned. This likely means that the time range is incorrect or that the data is not public. See the following for details:  https://cliflo.niwa.co.nz/pls/niwp/wh.do_help?id=nodata')
+            ts1['DatasetCode'] = dataset_code
+            ts2 = ts1.set_index(['Station', 'DatasetCode', 'Date(NZST)']).stack().reset_index()
+            ts2.rename(columns={'level_3': 'Parameter', 0: 'Value'}, inplace=True)
+
+            ts_df = ts2.copy()
+
+            ts_df['Date(NZST)'] = pd.to_datetime(ts_df['Date(NZST)'], format='%Y%m%d:%H%M')
+            ts_df.loc[ts_df.Value == '-', 'Value'] = np.nan
+
+            self.ts_data = ts_df
+
+            ## Number of rows left
+            rr_index = page.find('Number of rows remaining in subscription')
+            copy_index = page.find('Copyright NIWA')
+            rr = int(page[(rr_index + 43):(copy_index - 1)])
+            print('You have ' + str(rr) + ' remaining rows')
+            self.remaining_rows = rr
+
+            ts_list.append(ts_df)
+
+        if ts_list:
+            ts_df2 = pd.concat(ts_list)
         else:
-            ts_str = page_ts[site_index1:(site_index2 - 1)]
-            footer = 2
-
-        ts_str1 = StringIO(ts_str)
-        ts1 = pd.read_table(ts_str1, skipfooter=footer, engine='python')
-        if ts1.empty:
-            raise IndexError('No time series data was returned. This likely means that the time range is incorrect or that the data is not public. See the following for details:  https://cliflo.niwa.co.nz/pls/niwp/wh.do_help?id=nodata')
-        ts1['DatasetCode'] = dataset_code
-        ts2 = ts1.set_index(['Station', 'DatasetCode', 'Date(NZST)']).stack().reset_index()
-        ts2.rename(columns={'level_3': 'Parameter', 0: 'Value'}, inplace=True)
-
-        ts_df = ts2.copy()
-
-        ts_df['Date(NZST)'] = pd.to_datetime(ts_df['Date(NZST)'], format='%Y%m%d:%H%M')
-        ts_df.loc[ts_df.Value == '-', 'Value'] = np.nan
-
-        self.ts_data = ts_df
-
-        ## Number of rows left
-        rr_index = page.find('Number of rows remaining in subscription')
-        copy_index = page.find('Copyright NIWA')
-        rr = int(page[(rr_index + 43):(copy_index - 1)])
-        print('You have ' + str(rr) + ' remaining rows')
-        self.remaining_rows = rr
+            ts_df2 = pd.DataFrame()
 
         ## Return
-        return sites1, ts_df
+        return sites1, ts_df2
 
 
     @staticmethod
@@ -516,23 +553,3 @@ def generate_username(num_results=1):
         usernames.append(adjective + noun + num)
 
     return usernames
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
