@@ -4,7 +4,7 @@ Created on Tue Jul  2 09:25:41 2019
 
 @author: michaelek
 """
-import pytest
+# import pytest
 from pycliflo import Cliflo
 import pandas as pd
 
